
#define DBG_GUI		0

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "demo.h"
#include "my-pc.h"
#include "vuc.h"
#include "libvuc.h"


gchar * glade= "demo.glade";

gint scope_diag_ok= 0;
gint scope_diag_err= 0;


gboolean vuc_io_in (GIOChannel * source, GIOCondition condition, gpointer data);
gboolean backgr_func (gpointer data);

guint status_msg_id;


i8 scope_in (u8 * data);
void vuc_in_pkt_start (void);
void vuc_in_send_hs (i8 rep);


vuc_put_data_t vuc_put_meta[VUC_PC_PUT_ADRS]= {
  { SCOPE_SIZE, scope_in },
};

vuc_get_data_t vuc_get_meta[VUC_PC_GET_ADRS]= {
};


#if (VUC_DBG_ON)
#  define dbgf(...)	msgf(__VA_ARGS__)
#else
#  define dbgf(...)	
#endif

#if (DBG_GUI)
#  define dbgf4(...)	dbgf(__VA_ARGS__)
#else
#  define dbgf4(...)	
#endif


/*****************************************************************************/

int msgf (const char * fmt, ...)
{
  va_list args;

#if (SHOW_TIMESTAMP)
  i32 now= my_time();
  printf("%d.%03d ", now/1000, now%1000);
#endif

  va_start(args, fmt);
  vprintf(fmt, args);
  printf("\n");
  va_end(args);
}

int errf (const char * fmt, ...)
{
  va_list args;

#if (SHOW_TIMESTAMP)
  i32 now= my_time();
  printf("%d.%03d ", now/1000, now%1000);
#endif

  printf("ERROR! ");

  va_start(args, fmt);
  vprintf(fmt, args);
  printf("\n");
  va_end(args);
}


/*** Gen *********************************************************************/

GtkToggleButton * u_gen_power;
GtkAdjustment * u_gen_freq;
GtkAdjustment * u_gen_pw;

GtkDrawingArea * u_scope_area;
GtkToggleButton * u_scope_tb[8];

gdouble u_gen_freq_v= 20;
gdouble u_gen_pw_v= 1;

void u_gen_set ()
{
  u16 f= (u16)(F_CPU/8/2/u_gen_freq_v);
  u16 p= (u16)(f*(u_gen_pw_v/100));
  if (p==0)
    p++;
  if (p==f)
    p--;
  vuc_put_16(0, 2, f, p);
}

void on_u_gen_power_toggled (GtkWidget * widget, gpointer udata)
{
  u8 v= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  dbgf4("Power: %d", v);
  vuc_put_8(1, 1, v);
}

void on_u_gen_freq_value_changed (GtkWidget * widget, gpointer udata)
{
  u_gen_freq_v= gtk_adjustment_get_value(GTK_ADJUSTMENT(widget));
#if (DBG_GUI)
  dbgf4("Freq: %04.0f", u_gen_freq_v);
#endif
  u_gen_set();
}

void on_u_gen_pw_value_changed (GtkWidget * widget, gpointer udata)
{
  u_gen_pw_v= gtk_adjustment_get_value(GTK_ADJUSTMENT(widget));
#if (DBG_GUI)
  dbgf4("PW: %02.0f", u_gen_pw_v);
#endif
  u_gen_set();
}


/*** Scope *******************************************************************/

// Timebase constants in uS/div
gint sc_timebase[8]= {
  500, 1000, 2000, 5000, 10000, 20000, 50000, 100000
};

vuc_scope_t * u_scope;

i8 scope_in (u8 * data)
{
  u8 x, v;
  gint c, lc= 255-data[0];

#if (VUC_TEST_PROTO==1)
  static u8 diag_cnt= 0;
  gint fl= 0;

  for (x=0; x<SCOPE_SIZE; x++) {
    v= x ? x : diag_cnt++;
    c= data[x];
    if (c!=v) {
      printf(" !%02x:%02x ", v, c);
      fl= 1;
      if (!x)
        diag_cnt= c+1;
    }
  }
  if (fl) {
    scope_diag_err++;
    printf("\n");
  } else
    scope_diag_ok++;

  return 1;
#endif

  u_scope->count= 0;
  for (x=0; x<SCOPE_SIZE; x++) {
    c= 255-data[x];
    if (x==0 || c!=lc) {
      u_scope->points[u_scope->count].x= x*VUC_SCOPE_NORM/SCOPE_SIZE; u_scope->points[u_scope->count].y= lc*VUC_SCOPE_NORM/256; u_scope->count++;
      u_scope->points[u_scope->count].x= x*VUC_SCOPE_NORM/SCOPE_SIZE; u_scope->points[u_scope->count].y= c*VUC_SCOPE_NORM/256; u_scope->count++;
    }
    lc= c;
  }
  u_scope->points[u_scope->count].x= x*VUC_SCOPE_NORM/SCOPE_SIZE; u_scope->points[u_scope->count].y= c*VUC_SCOPE_NORM/256; u_scope->count++;
  vuc_scope_update(u_scope);
  return 1;
}

#if (VUC_TEST_PROTO==1)
FIXME!
  if (last_scope_diag_ok!=scope_diag_ok || last_scope_diag_err!=scope_diag_err) {
    last_scope_diag_ok= scope_diag_ok; last_scope_diag_err= scope_diag_err;
    snprintf(text, 32, "OK: %06d  ERR: %06d", scope_diag_ok, scope_diag_err);
    gtk_entry_set_text(gui_status_msg, text);
  }
#endif

gboolean in_scope_tb= 0;

gboolean on_u_scope_tb_toggled (GtkWidget * widget, gpointer udata)
{
  gint i, t, v= (int)udata;

  if (in_scope_tb)
    return FALSE;

  in_scope_tb= TRUE;
  for (i=0; i<8; i++) {
    gtk_toggle_button_set_active(u_scope_tb[i], i==v);
  }
  in_scope_tb= FALSE;

  t= (F_CPU/SCOPE_PRESC_VAL) / (1000000/(sc_timebase[v]*8/SCOPE_SIZE));
  dbgf("Timebase: %d -> %5.1f mS/div -> t=%d", (int)udata, sc_timebase[v]/1000.0, t);
  vuc_put_8(2, 1, t);

  return FALSE;
}


/* main **********************************************************************/

int main (int argc, char * argv[])
{
  gint i;
  gchar * contents;
  GtkBuilder * builder;

  GtkScrolledWindow * dbg_scr;
  GtkAdjustment * dbg_adj;
  GtkTextView * dbg_buf;

  GThread * send_th;

  GtkStyle * style;

  if (!g_file_get_contents(glade, &contents, NULL, NULL))
    exit(1);

  builder= vuc_init(BAUDRATE, -1, contents, VUC_PC_PUT_ADRS, vuc_put_meta, VUC_PC_GET_ADRS, vuc_get_meta, argc, argv);

  u_gen_power= GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "u_gen_power"));
  gtk_signal_connect(GTK_OBJECT(u_gen_power), "toggled", GTK_SIGNAL_FUNC(on_u_gen_power_toggled), NULL);

  u_gen_freq= GTK_ADJUSTMENT(gtk_builder_get_object(builder, "u_gen_freq"));
  gtk_adjustment_set_value(u_gen_freq, 0);
  gtk_signal_connect(GTK_OBJECT(u_gen_freq), "value-changed", GTK_SIGNAL_FUNC(on_u_gen_freq_value_changed), NULL);

  u_gen_pw= GTK_ADJUSTMENT(gtk_builder_get_object(builder, "u_gen_pw"));
  gtk_adjustment_set_value(u_gen_pw, 0);
  gtk_signal_connect(GTK_OBJECT(u_gen_pw), "value-changed", GTK_SIGNAL_FUNC(on_u_gen_pw_value_changed), NULL);

  u_scope= vuc_scope_init(SCOPE_SIZE, "u_scope_area");

  for (i=0; i<8; i++) {
    char buf[20];
    sprintf(buf, "u_scope_tb_%d", i+1);
    u_scope_tb[i]= GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, buf));
    gtk_signal_connect(GTK_OBJECT(u_scope_tb[i]), "toggled", GTK_SIGNAL_FUNC(on_u_scope_tb_toggled), (void *)i);
  }

  vuc_go();

  return 0;
}
